@@ -14,17 +14,17 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Id: check_nagios.c,v 1.26 2005/05/25 00:43:20 seanius Exp $
+ $Id: check_nagios.c,v 1.29 2005/12/02 22:28:06 tonvoon Exp $
  
 ******************************************************************************/
 
 const char *progname = "check_nagios";
-const char *revision = "$Revision: 1.26 $";
+const char *revision = "$Revision: 1.29 $";
 const char *copyright = "1999-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
 #include "common.h"
-#include "popen.h"
+#include "runcmd.h"
 #include "utils.h"
 
 int process_arguments (int, char **);
@@ -55,24 +55,28 @@ main (int argc, char **argv)
 	int procrss = 0;
 	float procpcpu = 0;
 	char procstat[8];
+#ifdef PS_USES_PROCETIME
 	char procetime[MAX_INPUT_BUFFER];
+#endif /* PS_USES_PROCETIME */
 	char procprog[MAX_INPUT_BUFFER];
 	char *procargs;
 	int pos, cols;
 	int expected_cols = PS_COLS - 1;
 	const char *zombie = "Z";
 	char *temp_string;
+	output chld_out, chld_err;
+	size_t i;
 
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
 	if (process_arguments (argc, argv) == ERROR)
-		usage4 (_("Could not parse arguments"));
+		usage_va(_("Could not parse arguments"));
 
 	/* Set signal handling and alarm timeout */
 	if (signal (SIGALRM, timeout_alarm_handler) == SIG_ERR) {
-		usage4 (_("Cannot catch SIGALRM"));
+		usage_va(_("Cannot catch SIGALRM"));
 	}
 
 	/* handle timeouts gracefully... */
@@ -81,58 +85,51 @@ main (int argc, char **argv)
 	/* open the status log */
 	fp = fopen (status_log, "r");
 	if (fp == NULL) {
-		printf (_("CRITICAL - Cannot open status log for reading!\n"));
-		return STATE_CRITICAL;
+		die (STATE_CRITICAL, "NAGIOS %s: %s\n", _("CRITICAL"), _("Cannot open status log for reading!"));
 	}
 
 	/* get the date/time of the last item updated in the log */
 	while (fgets (input_buffer, MAX_INPUT_BUFFER - 1, fp)) {
-		temp_ptr = strtok (input_buffer, "]");
-		temp_entry_time =
-			(temp_ptr == NULL) ? 0L : strtoul (temp_ptr + 1, NULL, 10);
-		if (temp_entry_time > latest_entry_time)
+		if ((temp_ptr = strstr (input_buffer, "created=")) != NULL) {
+			temp_entry_time = strtoul (temp_ptr + 8, NULL, 10);
 			latest_entry_time = temp_entry_time;
+			break;
+		} else if ((temp_ptr = strtok (input_buffer, "]")) != NULL) {
+			temp_entry_time = strtoul (temp_ptr + 1, NULL, 10);
+			if (temp_entry_time > latest_entry_time)
+				latest_entry_time = temp_entry_time;
+		}
 	}
 	fclose (fp);
 
 	if (verbose >= 2)
-		printf(_("command: %s\n"), PS_COMMAND);
+		printf("command: %s\n", PS_COMMAND);
 
 	/* run the command to check for the Nagios process.. */
-	child_process = spopen (PS_COMMAND);
-	if (child_process == NULL) {
-		printf (_("Could not open pipe: %s\n"), PS_COMMAND);
-		return STATE_UNKNOWN;
-	}
-
-	child_stderr = fdopen (child_stderr_array[fileno (child_process)], "r");
-	if (child_stderr == NULL) {
-		printf (_("Could not open stderr for %s\n"), PS_COMMAND);
-	}
-
-	fgets (input_buffer, MAX_INPUT_BUFFER - 1, child_process);
+	if((result = np_runcmd(PS_COMMAND, &chld_out, &chld_err, 0)) != 0)
+		result = STATE_WARNING;
 
 	/* count the number of matching Nagios processes... */
-	while (fgets (input_buffer, MAX_INPUT_BUFFER - 1, child_process)) {
-		cols = sscanf (input_buffer, PS_FORMAT, PS_VARLIST);
-                /* Zombie processes do not give a procprog command */
-                if ( cols == (expected_cols - 1) && strstr(procstat, zombie) ) {
-                        cols = expected_cols;
-                        /* Set some value for procargs for the strip command further below
-                        Seen to be a problem on some Solaris 7 and 8 systems */
-                        input_buffer[pos] = '\n';
-                        input_buffer[pos+1] = 0x0;
-                }
+	for(i = 0; i < chld_out.lines; i++) {
+		cols = sscanf (chld_out.line[i], PS_FORMAT, PS_VARLIST);
+		/* Zombie processes do not give a procprog command */
+		if ( cols == (expected_cols - 1) && strstr(procstat, zombie) ) {
+			cols = expected_cols;
+			/* Set some value for procargs for the strip command further below
+			 * Seen to be a problem on some Solaris 7 and 8 systems */
+			chld_out.line[i][pos] = '\n';
+			chld_out.line[i][pos+1] = 0x0;
+		}
 		if ( cols >= expected_cols ) {
-			asprintf (&procargs, "%s", input_buffer + pos);
+			asprintf (&procargs, "%s", chld_out.line[i] + pos);
 			strip (procargs);
-			
+
 			/* Some ps return full pathname for command. This removes path */
-                        temp_string = strtok ((char *)procprog, "/");
-                        while (temp_string) {
-                                strcpy(procprog, temp_string);
-                                temp_string = strtok (NULL, "/");
-                        }
+			temp_string = strtok ((char *)procprog, "/");
+			while (temp_string) {
+				strcpy(procprog, temp_string);
+				temp_string = strtok (NULL, "/");
+			}
 
 			/* May get empty procargs */
 			if (!strstr(procargs, argv[0]) && strstr(procargs, process_string) && strcmp(procargs,"")) {
@@ -145,36 +142,36 @@ main (int argc, char **argv)
 	}
 
 	/* If we get anything on stderr, at least set warning */
-	while (fgets (input_buffer, MAX_INPUT_BUFFER - 1, child_stderr))
-		result = max_state (result, STATE_WARNING);
-
-	/* close stderr */
-	(void) fclose (child_stderr);
-
-	/* close the pipe */
-	if (spclose (child_process))
+	if(chld_err.buflen)
 		result = max_state (result, STATE_WARNING);
 
 	/* reset the alarm handler */
 	alarm (0);
 
 	if (proc_entries == 0) {
-		printf (_("Could not locate a running Nagios process!\n"));
-		return STATE_CRITICAL;
+		die (STATE_CRITICAL, "NAGIOS %s: %s\n", _("CRITICAL"), _("Could not locate a running Nagios process!"));
 	}
 
-	result = STATE_OK;
+	if (latest_entry_time == 0L) {
+		die (STATE_CRITICAL, "NAGIOS %s: %s\n", _("CRITICAL"), _("Cannot parse Nagios log file for valid time"));
+	}
 
 	time (&current_time);
-	if ((int)(current_time - latest_entry_time) > (expire_minutes * 60))
+	if ((int)(current_time - latest_entry_time) > (expire_minutes * 60)) {
 		result = STATE_WARNING;
+	} else {
+		result = STATE_OK;
+	}
 
-	printf
-		(_("Nagios %s: located %d process%s, status log updated %d second%s ago\n"),
-		 (result == STATE_OK) ? "ok" : "problem", proc_entries,
-		 (proc_entries == 1) ? "" : "es",
-		 (int) (current_time - latest_entry_time),
-		 ((int) (current_time - latest_entry_time) == 1) ? "" : "s");
+	printf ("NAGIOS %s: ", (result == STATE_OK) ? _("OK") : _("WARNING"));
+	printf (ngettext ("%d process", "%d processes", proc_entries), proc_entries);
+	printf (", ");
+	printf (
+	  ngettext ("status log updated %d second ago", 
+	    "status log updated %d seconds ago", 
+	    (int) (current_time - latest_entry_time) ),
+	    (int) (current_time - latest_entry_time) );
+	printf ("\n");
 
 	return result;
 }
@@ -219,8 +216,6 @@ process_arguments (int argc, char **argv)
 			break;
 
 		switch (c) {
-		case '?':									/* print short usage statement if args not parsable */
-			usage2 (_("Unknown argument"), optarg);
 		case 'h':									/* help */
 			print_help ();
 			exit (STATE_OK);
@@ -243,16 +238,17 @@ process_arguments (int argc, char **argv)
 		case 'v':
 			verbose++;
 			break;
+		default:									/* print short usage_va statement if args not parsable */
+			usage_va(_("Unknown argument - %s"), optarg);
 		}
 	}
 
 
 	if (status_log == NULL)
-		die (STATE_UNKNOWN,
-		     _("You must provide the status_log\n"));
-	else if (process_string == NULL)
-		die (STATE_UNKNOWN,
-							 _("You must provide a process string\n"));
+		die (STATE_UNKNOWN, _("You must provide the status_log\n"));
+
+	if (process_string == NULL)
+		die (STATE_UNKNOWN, _("You must provide a process string\n"));
 
 	return OK;
 }
