@@ -14,12 +14,12 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Id: check_ping.c,v 1.49 2005/09/14 14:40:20 tonvoon Exp $
+ $Id: check_ping.c,v 1.50 2006/03/13 11:08:28 tonvoon Exp $
  
 ******************************************************************************/
 
 const char *progname = "check_ping";
-const char *revision = "$Revision: 1.49 $";
+const char *revision = "$Revision: 1.50 $";
 const char *copyright = "2000-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -452,9 +452,22 @@ run_ping (const char *cmd, const char *addr)
 		rta = crta;
 
 	/* check stderr, setting at least WARNING if there is output here */
-	while (fgets (buf, MAX_INPUT_BUFFER - 1, child_stderr))
-		if (! strstr(buf,"WARNING - no SO_TIMESTAMP support, falling back to SIOCGSTAMP"))
-			result = max_state (STATE_WARNING, error_scan (buf, addr));
+	/* Add warning into warn_text */
+	while (fgets (buf, MAX_INPUT_BUFFER - 1, child_stderr)) {
+		if (! strstr(buf,"WARNING - no SO_TIMESTAMP support, falling back to SIOCGSTAMP")) {
+			if (verbose >= 3) {
+				printf("Got stderr: %s", buf);
+			}
+			if ((result=error_scan(buf, addr)) == STATE_OK) {
+				result = STATE_WARNING;
+				if (warn_text == NULL) {
+					warn_text = strdup(_("System call sent warnings to stderr "));
+				} else {
+					asprintf(&warn_text, "%s %s", warn_text, _("System call sent warnings to stderr "));
+				}
+			}
+		}
+	}
 
 	(void) fclose (child_stderr);
 
