@@ -14,12 +14,12 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Id: check_procs.c,v 1.46 2005/09/15 08:27:58 tonvoon Exp $
+ $Id: check_procs.c,v 1.48 2006/04/05 08:06:19 tonvoon Exp $
  
 ******************************************************************************/
 
 const char *progname = "check_procs";
-const char *revision = "$Revision: 1.46 $";
+const char *revision = "$Revision: 1.48 $";
 const char *copyright = "2000-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -65,7 +65,7 @@ enum metric metric = METRIC_PROCS;
 
 int verbose = 0;
 int uid;
-int ppid;
+pid_t ppid;
 int vsz;
 int rss;
 float pcpu;
@@ -87,8 +87,8 @@ main (int argc, char **argv)
 
 	pid_t mypid = 0;
 	int procuid = 0;
-	int procpid = 0;
-	int procppid = 0;
+	pid_t procpid = 0;
+	pid_t procppid = 0;
 	int procvsz = 0;
 	int procrss = 0;
 	int procseconds = 0;
@@ -116,6 +116,7 @@ main (int argc, char **argv)
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
+	setlocale(LC_NUMERIC, "POSIX");
 
 	input_buffer = malloc (MAX_INPUT_BUFFER);
 	procprog = malloc (MAX_INPUT_BUFFER);
