@@ -14,12 +14,12 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Id: check_snmp.c,v 1.57 2005/06/01 19:41:01 sghosh Exp $
+ $Id: check_snmp.c,v 1.58 2005/10/31 20:03:19 seanius Exp $
  
 ******************************************************************************/
 
 const char *progname = "check_snmp";
-const char *revision = "$Revision: 1.57 $";
+const char *revision = "$Revision: 1.58 $";
 const char *copyright = "1999-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -578,7 +578,7 @@ process_arguments (int argc, char **argv)
 				labels_size += 8;
 				labels = realloc (labels, labels_size);
 				if (labels == NULL)
-					die (STATE_UNKNOWN, _("Could not reallocate labels[%d]"), nlabels);
+					die (STATE_UNKNOWN, _("Could not reallocate labels[%d]"), (int)nlabels);
 			}
 			labels[nlabels - 1] = optarg;
 			ptr = thisarg (optarg);
@@ -607,7 +607,7 @@ process_arguments (int argc, char **argv)
 				unitv_size += 8;
 				unitv = realloc (unitv, unitv_size);
 				if (unitv == NULL)
-					die (STATE_UNKNOWN, _("Could not reallocate units [%d]\n"), nunits);
+					die (STATE_UNKNOWN, _("Could not reallocate units [%d]\n"), (int)nunits);
 			}
 			unitv[nunits - 1] = optarg;
 			ptr = thisarg (optarg);
