@@ -14,12 +14,12 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Id: check_time.c,v 1.25 2004/12/25 23:17:44 opensides Exp $
+ $Id: check_time.c,v 1.26 2005/05/25 17:42:35 sghosh Exp $
  
 ******************************************************************************/
 
 const char *progname = "check_time";
-const char *revision = "$Revision: 1.25 $";
+const char *revision = "$Revision: 1.26 $";
 const char *copyright = "1999-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -33,7 +33,7 @@ enum {
 
 #define	UNIX_EPOCH 2208988800UL
 
-unsigned long server_time, raw_server_time;
+uint32_t server_time, raw_server_time;
 time_t diff_time;
 int warning_time = 0;
 int check_warning_time = FALSE;
