@@ -28,10 +28,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: common.h,v 1.15 2004/12/10 00:13:43 tonvoon Exp $
+ * $Id: common.h,v 1.18 2005/12/07 15:10:27 tonvoon Exp $
  *
  *****************************************************************************/
 
+#ifndef _COMMON_H_
+#define _COMMON_H_
+
 #include "config.h"
 
 #ifdef HAVE_FEATURES_H
@@ -42,7 +45,7 @@
 #include <stdlib.h>
 #include <errno.h>
 
-#ifdef HUGE_VAL_NEEDS_MATH_H
+#ifdef HAVE_MATH_H
 #include <math.h>
 #endif
 
@@ -146,6 +149,29 @@ int snprintf(char *str, size_t size, const  char  *format, ...);
 int vsnprintf(char *str, size_t size, const char  *format, va_list ap);
 #endif
 
+/* SSL implementations */
+#ifdef HAVE_GNUTLS_OPENSSL_H
+#  include <gnutls/openssl.h>
+#else
+#  ifdef HAVE_SSL_H
+#    include <rsa.h>
+#    include <crypto.h>
+#    include <x509.h>
+#    include <pem.h>
+#    include <ssl.h>
+#    include <err.h>
+#  else
+#    ifdef HAVE_OPENSSL_SSL_H
+#      include <openssl/rsa.h>
+#      include <openssl/crypto.h>
+#      include <openssl/x509.h>
+#      include <openssl/pem.h>
+#      include <openssl/ssl.h>
+#      include <openssl/err.h>
+#    endif
+#  endif
+#endif
+
 /*
  *
  * Standard Values
@@ -165,6 +191,11 @@ enum {
 };
 #endif
 
+/* Solaris does not have floorf, but floor works. Should probably be in configure */
+#if defined(__sun) || defined(__sun__)
+static inline float floorf (float x) { return floor(x); }
+#endif
+
 enum {
 	STATE_OK,
 	STATE_WARNING,
@@ -191,3 +222,5 @@ enum {
 #ifndef __GNUC__
 # define __attribute__(x) /* do nothing */
 #endif
+
+#endif /* _COMMON_H_ */
