@@ -16,7 +16,7 @@ int spclose(FILE *);
  * path passed to the exec'd program are esstially empty. (popen create a shell
  * and passes the environment to it).
  *
- * $Id: popen.c,v 1.13 2005/09/12 10:31:28 tonvoon Exp $
+ * $Id: popen.c,v 1.14 2006/02/17 09:08:08 tonvoon Exp $
  *
  ******************************************************************************/
 
@@ -270,12 +270,16 @@ popen_timeout_alarm_handler (int signo)
 {
 	int fh;
 	if (signo == SIGALRM) {
-		fh=fileno (child_process);
-		if(fh >= 0){
-			kill (childpid[fh], SIGKILL);
-		}
-		printf (_("CRITICAL - Plugin timed out after %d seconds\n"),
+		if (child_process != NULL) {
+			fh=fileno (child_process);
+			if(fh >= 0){
+				kill (childpid[fh], SIGKILL);
+			}
+			printf (_("CRITICAL - Plugin timed out after %d seconds\n"),
 						timeout_interval);
+		} else {
+			printf (_("CRITICAL - popen timeout received, but no child process\n"));
+		}
 		exit (STATE_CRITICAL);
 	}
 }
