@@ -1,3 +1,5 @@
+#ifndef NP_UTILS_H
+#define NP_UTILS_H
 /* Header file for nagios plugins utils.c */
 
 /* This file should be included in all plugins */
@@ -13,7 +15,7 @@ suite of plugins. */
 
 /* Standardize version information, termination */
 
-/* $Id: utils.h,v 1.22 2004/12/05 00:54:09 opensides Exp $ */
+/* $Id: utils.h,v 1.28 2006/03/29 16:33:36 tonvoon Exp $ */
 
 void support (void);
 char *clean_revstring (const char *);
@@ -56,6 +58,28 @@ struct timeval {
 };
 #endif
 
+#define OUTSIDE 0
+#define INSIDE  1
+
+typedef struct range_struct {
+	double	start;
+	int	start_infinity;		/* FALSE (default) or TRUE */
+	double	end;
+	int	end_infinity;
+	int	alert_on;		/* OUTSIDE (default) or INSIDE */
+	} range;
+
+typedef struct thresholds_struct {
+	range	*warning;
+	range	*critical;
+	} thresholds;
+
+range *parse_range_string (char *);
+int _set_thresholds(thresholds **, char *, char *);
+void set_thresholds(thresholds **, char *, char *);
+int check_range(double, range *);
+int get_status(double, thresholds *);
+
 #ifndef HAVE_GETTIMEOFDAY
 int gettimeofday(struct timeval *, struct timezone *);
 #endif
@@ -77,6 +101,7 @@ void usage (const char *) __attribute__((noreturn));
 void usage2(const char *, const char *) __attribute__((noreturn));
 void usage3(const char *, int) __attribute__((noreturn));
 void usage4(const char *);
+void usage_va(const char *fmt, ...);
 
 const char *state_text (int);
 
@@ -107,6 +132,8 @@ char *fperfdata (const char *,
  int,
  double);
 
+char *np_escaped_string (const char *);
+
 /* The idea here is that, although not every plugin will use all of these, 
    most will or should.  Therefore, for consistency, these very common 
    options should have only these meanings throughout the overall suite */
@@ -136,7 +163,7 @@ char *fperfdata (const char *,
 
 #define UT_HOST_PORT "\
  -H, --hostname=ADDRESS\n\
-    Host name or IP Address\n\
+    Host name, IP Address, or unix socket (must be an absolute path)\n\
  -%c, --port=INTEGER\n\
     Port number (default: %s)\n"
 
@@ -156,6 +183,12 @@ char *fperfdata (const char *,
  -c, --critical=DOUBLE\n\
     Response time to result in critical status (seconds)\n"
 
+#define UT_WARN_CRIT_RANGE "\
+ -w, --warning=RANGE\n\
+    Warning range (format: start:end). Alert if outside this range\n\
+ -c, --critical=RANGE\n\
+    Critical range\n"
+
 #define UT_TIMEOUT "\
  -t, --timeout=INTEGER\n\
     Seconds before connection times out (default: %d)\n"
@@ -169,3 +202,5 @@ send email to nagiosplug-devel@lists.sourceforge.net\n"
 The nagios plugins come with ABSOLUTELY NO WARRANTY. You may redistribute\n\
 copies of the plugins under the terms of the GNU General Public License.\n\
 For more information about these matters, see the file named COPYING.\n"
+
+#endif /* NP_UTILS_H */
